# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CESNET.
#
# CESNET-OpenID-Remote is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""CESNET OIDC Auth backend for OARepo"""

OPENIDC_CONFIG = 'CESNET_OPENIDC_CONFIG'
OPENIDC_GROUPS_SCOPE = 'eduperson_entitlement_extended'
OPENIDC_BASE_URL = 'https://login.cesnet.cz/oidc/'
OPENIDC_GROUPS_KEY = 'cesnet_groups'
