# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CESNET.
#
# CESNET-OpenID-Remote is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Version information for CESNET-OpenID-Remote.

This file is imported by ``cesnet_openid_remote.__init__``,
and parsed by ``setup.py``.
"""

__version__ = '2.0.1'
